@@ -60,6 +60,8 @@ struct _GfLogicalMonitor
 };
 
 typedef void (* GfLogicalMonitorCrtcFunc) (GfLogicalMonitor *logical_monitor,
+                                           GfMonitor        *monitor,
+                                           GfOutput         *output,
                                            GfCrtc           *crtc,
                                            gpointer          user_data);
 
