@@ -100,17 +100,23 @@
 
 struct _GfMonitorConfigStore
 {
-  GObject           parent;
+  GObject                parent;
 
-  GfMonitorManager *monitor_manager;
+  GfMonitorManager      *monitor_manager;
 
-  GHashTable       *configs;
+  GHashTable            *configs;
 
-  GCancellable     *save_cancellable;
+  GCancellable          *save_cancellable;
 
-  GFile            *user_file;
-  GFile            *custom_read_file;
-  GFile            *custom_write_file;
+  GFile                 *user_file;
+  GFile                 *custom_read_file;
+  GFile                 *custom_write_file;
+
+  gboolean               has_stores_policy;
+  GList                 *stores_policy;
+
+  gboolean               has_dbus_policy;
+  GfMonitorConfigPolicy  policy;
 };
 
 enum
@@ -127,6 +133,7 @@ G_DEFINE_QUARK (gf-monitor-config-store-error-quark,
 typedef enum
 {
   STATE_INITIAL,
+  STATE_UNKNOWN,
   STATE_MONITORS,
   STATE_CONFIGURATION,
   STATE_MIGRATED,
@@ -151,13 +158,27 @@ typedef enum
   STATE_MONITOR_MODE_RATE,
   STATE_MONITOR_MODE_FLAG,
   STATE_MONITOR_UNDERSCANNING,
-  STATE_DISABLED
+  STATE_DISABLED,
+  STATE_POLICY,
+  STATE_STORES,
+  STATE_STORE,
+  STATE_DBUS
 } ParserState;
 
+typedef enum
+{
+  GF_CONFIG_STORE_NONE,
+  GF_CONFIG_STORE_SYSTEM,
+  GF_CONFIG_STORE_USER,
+} GfConfigStore;
+
 typedef struct
 {
   ParserState             state;
   GfMonitorConfigStore   *config_store;
+  GFile                  *file;
+
+  GHashTable             *pending_configs;
 
   ParserState             monitor_spec_parent_state;
 
@@ -171,6 +192,18 @@ typedef struct
   GfLogicalMonitorConfig *current_logical_monitor_config;
   GList                  *current_disabled_monitor_specs;
 
+  gboolean                seen_policy;
+  gboolean                seen_stores;
+  gboolean                seen_dbus;
+  GfConfigStore           pending_store;
+  GList                  *stores;
+
+  gboolean                enable_dbus_set;
+  gboolean                enable_dbus;
+
+  ParserState             unknown_state_root;
+  int                     unknown_level;
+
   GfMonitorsConfigFlag    extra_config_flags;
 } ConfigParser;
 
@@ -193,12 +226,55 @@ static GParamSpec *config_store_properties[LAST_PROP] = { NULL };
 
 G_DEFINE_TYPE (GfMonitorConfigStore, gf_monitor_config_store, G_TYPE_OBJECT)
 
+static void
+replace_configs (GfMonitorConfigStore *config_store,
+                 GHashTable           *configs)
+{
+  GHashTableIter iter;
+  GfMonitorsConfigKey *key;
+  GfMonitorsConfig *config;
+
+  g_hash_table_iter_init (&iter, configs);
+  while (g_hash_table_iter_next (&iter,
+                                 (gpointer *) &key,
+                                 (gpointer *) &config))
+    {
+      g_hash_table_iter_steal (&iter);
+      g_hash_table_replace (config_store->configs, key, config);
+    }
+}
+
 static gboolean
 is_system_config (GfMonitorsConfig *config)
 {
   return !!(config->flags & GF_MONITORS_CONFIG_FLAG_SYSTEM_CONFIG);
 }
 
+static gboolean
+text_equals (const char *text,
+             size_t      len,
+             const char *expect)
+{
+  if (strlen (expect) != len)
+    return FALSE;
+
+  return strncmp (text, expect, len) == 0;
+}
+
+static void
+enter_unknown_element (ConfigParser *parser,
+                       const char   *element_name,
+                       const char   *root_element_name,
+                       ParserState   root_state)
+{
+  parser->state = STATE_UNKNOWN;
+  parser->unknown_level = 1;
+  parser->unknown_state_root = root_state;
+
+  g_warning ("Unknown element <%s> under <%s>, ignoring",
+             element_name, root_element_name);
+}
+
 static void
 handle_start_element (GMarkupParseContext  *context,
                       const gchar          *element_name,
@@ -253,15 +329,39 @@ handle_start_element (GMarkupParseContext  *context,
 
       case STATE_MONITORS:
         {
-          if (!g_str_equal (element_name, "configuration"))
+          if (g_str_equal (element_name, "configuration"))
             {
-              g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
-                           "Invalid toplevel element '%s'", element_name);
+              parser->state = STATE_CONFIGURATION;
+              parser->current_was_migrated = FALSE;
+            }
+          else if (g_str_equal (element_name, "policy"))
+            {
+              if (parser->seen_policy)
+                {
+                  g_set_error (error,
+                               G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                               "Multiple policy definitions");
+                  return;
+                }
+
+              parser->seen_policy = TRUE;
+              parser->state = STATE_POLICY;
+            }
+          else
+            {
+              enter_unknown_element (parser,
+                                     element_name,
+                                     "monitors",
+                                     STATE_MONITORS);
               return;
             }
 
-          parser->state = STATE_CONFIGURATION;
-          parser->current_was_migrated = FALSE;
+          return;
+        }
+
+      case STATE_UNKNOWN:
+        {
+          parser->unknown_level++;
 
           return;
         }
@@ -286,9 +386,10 @@ handle_start_element (GMarkupParseContext  *context,
             }
           else
             {
-              g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
-                           "Invalid configuration element '%s'", element_name);
-              return;
+              enter_unknown_element (parser,
+                                     element_name,
+                                     "configuration",
+                                     STATE_CONFIGURATION);
             }
 
           return;
@@ -335,9 +436,10 @@ handle_start_element (GMarkupParseContext  *context,
             }
           else
             {
-              g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
-                           "Invalid monitor logicalmonitor element '%s'", element_name);
-              return;
+              enter_unknown_element (parser,
+                                     element_name,
+                                     "logicalmonitor",
+                                     STATE_LOGICAL_MONITOR);
             }
 
           return;
@@ -504,6 +606,82 @@ handle_start_element (GMarkupParseContext  *context,
           return;
         }
 
+      case STATE_POLICY:
+        {
+          if (!(parser->extra_config_flags & GF_MONITORS_CONFIG_FLAG_SYSTEM_CONFIG))
+            {
+              g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                           "Policy can only be defined in system level configurations");
+              return;
+            }
+
+          if (g_str_equal (element_name, "stores"))
+            {
+              if (parser->seen_stores)
+                {
+                  g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                               "Multiple stores elements under policy");
+                  return;
+                }
+
+              parser->seen_stores = TRUE;
+              parser->state = STATE_STORES;
+            }
+          else if (g_str_equal (element_name, "dbus"))
+            {
+              if (parser->seen_dbus)
+                {
+                  g_set_error (error,
+                               G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                               "Multiple dbus elements under policy");
+                  return;
+                }
+
+              parser->seen_dbus = TRUE;
+              parser->state = STATE_DBUS;
+            }
+          else
+            {
+              enter_unknown_element (parser,
+                                     element_name,
+                                     "policy",
+                                     STATE_POLICY);
+            }
+
+          return;
+        }
+
+      case STATE_STORES:
+        {
+          if (g_str_equal (element_name, "store"))
+            {
+              parser->state = STATE_STORE;
+            }
+          else
+            {
+              enter_unknown_element (parser,
+                                     element_name,
+                                     "stores",
+                                     STATE_STORES);
+            }
+
+          return;
+        }
+
+      case STATE_STORE:
+        {
+          g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                       "Invalid store sub element '%s'", element_name);
+          return;
+        }
+
+      case STATE_DBUS:
+        {
+          g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                       "Invalid dbus sub element '%s'", element_name);
+          return;
+        }
+
       default:
         break;
     }
@@ -589,6 +767,7 @@ finish_monitor_spec (ConfigParser *parser)
         }
 
       case STATE_INITIAL:
+      case STATE_UNKNOWN:
       case STATE_MONITORS:
       case STATE_CONFIGURATION:
       case STATE_MIGRATED:
@@ -612,6 +791,10 @@ finish_monitor_spec (ConfigParser *parser)
       case STATE_MONITOR_MODE_RATE:
       case STATE_MONITOR_MODE_FLAG:
       case STATE_MONITOR_UNDERSCANNING:
+      case STATE_POLICY:
+      case STATE_STORES:
+      case STATE_STORE:
+      case STATE_DBUS:
       default:
         g_assert_not_reached ();
         break;
@@ -822,13 +1005,94 @@ handle_end_element (GMarkupParseContext  *context,
               return;
             }
 
-          g_hash_table_replace (parser->config_store->configs,
+          g_hash_table_replace (parser->pending_configs,
                                 config->key, config);
 
           parser->state = STATE_MONITORS;
           return;
         }
 
+      case STATE_STORE:
+        g_assert (g_str_equal (element_name, "store"));
+
+        if (parser->pending_store == GF_CONFIG_STORE_NONE)
+          {
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                         "Got an empty store");
+            return;
+          }
+
+        if (g_list_find (parser->stores,
+                         GINT_TO_POINTER (parser->pending_store)))
+          {
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                         "Multiple identical stores in policy");
+            return;
+          }
+
+        parser->stores =
+          g_list_append (parser->stores,
+                         GINT_TO_POINTER (parser->pending_store));
+        parser->pending_store = GF_CONFIG_STORE_NONE;
+
+        parser->state = STATE_STORES;
+        return;
+
+      case STATE_STORES:
+        g_assert (g_str_equal (element_name, "stores"));
+
+        if (parser->config_store->has_stores_policy)
+          {
+            g_warning ("Ignoring stores policy from '%s', "
+                       "it has already been configured",
+                       g_file_peek_path (parser->file));
+            g_clear_pointer (&parser->stores, g_list_free);
+          }
+        else
+          {
+            parser->config_store->has_stores_policy = TRUE;
+            parser->config_store->stores_policy = parser->stores;
+            parser->stores = NULL;
+          }
+
+        parser->state = STATE_POLICY;
+        return;
+
+      case STATE_DBUS:
+        if (!parser->config_store->has_dbus_policy)
+          {
+            parser->config_store->has_dbus_policy = TRUE;
+            parser->config_store->policy.enable_dbus = parser->enable_dbus;
+            parser->enable_dbus_set = FALSE;
+          }
+        else
+          {
+            g_warning ("Policy for monitor configuration via D-Bus "
+                       "has already been set, ignoring policy from '%s'",
+                       g_file_get_path (parser->file));
+          }
+
+        parser->state = STATE_POLICY;
+        return;
+
+      case STATE_POLICY:
+        g_assert (g_str_equal (element_name, "policy"));
+
+        parser->state = STATE_MONITORS;
+        return;
+
+      case STATE_UNKNOWN:
+        {
+          parser->unknown_level--;
+          if (parser->unknown_level == 0)
+            {
+              g_assert (parser->unknown_state_root >= 0);
+              parser->state = parser->unknown_state_root;
+              parser->unknown_state_root = -1;
+            }
+          return;
+        }
+
       case STATE_MONITORS:
         {
           g_assert (g_str_equal (element_name, "monitors"));
@@ -907,12 +1171,12 @@ read_bool (const gchar  *text,
            gboolean     *out_value,
            GError      **error)
 {
-  if (strncmp (text, "no", text_len) == 0)
+  if (text_equals (text, text_len, "no"))
     {
       *out_value = FALSE;
       return TRUE;
     }
-  else if (strncmp (text, "yes", text_len) == 0)
+  else if (text_equals (text, text_len, "yes"))
     {
       *out_value = TRUE;
       return TRUE;
@@ -949,6 +1213,9 @@ handle_text (GMarkupParseContext  *context,
 
   switch (parser->state)
     {
+      case STATE_UNKNOWN:
+        return;
+
       case STATE_INITIAL:
       case STATE_MONITORS:
       case STATE_CONFIGURATION:
@@ -959,6 +1226,8 @@ handle_text (GMarkupParseContext  *context,
       case STATE_MONITOR_MODE:
       case STATE_TRANSFORM:
       case STATE_DISABLED:
+      case STATE_POLICY:
+      case STATE_STORES:
         {
           if (!is_all_whitespace (text, text_len))
             g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -1039,13 +1308,13 @@ handle_text (GMarkupParseContext  *context,
 
       case STATE_TRANSFORM_ROTATION:
         {
-          if (strncmp (text, "normal", text_len) == 0)
+          if (text_equals (text, text_len, "normal"))
             parser->current_transform = GF_MONITOR_TRANSFORM_NORMAL;
-          else if (strncmp (text, "left", text_len) == 0)
+          else if (text_equals (text, text_len, "left"))
             parser->current_transform = GF_MONITOR_TRANSFORM_90;
-          else if (strncmp (text, "upside_down", text_len) == 0)
+          else if (text_equals (text, text_len, "upside_down"))
             parser->current_transform = GF_MONITOR_TRANSFORM_180;
-          else if (strncmp (text, "right", text_len) == 0)
+          else if (text_equals (text, text_len, "right"))
             parser->current_transform = GF_MONITOR_TRANSFORM_270;
           else
             g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -1088,7 +1357,7 @@ handle_text (GMarkupParseContext  *context,
 
       case STATE_MONITOR_MODE_FLAG:
         {
-          if (strncmp (text, "interlace", text_len) == 0)
+          if (text_equals (text, text_len, "interlace"))
             {
               parser->current_monitor_mode_spec->flags |= GF_CRTC_MODE_FLAG_INTERLACE;
             }
@@ -1109,6 +1378,45 @@ handle_text (GMarkupParseContext  *context,
           return;
         }
 
+      case STATE_STORE:
+        {
+          GfConfigStore store;
+
+          if (parser->pending_store != GF_CONFIG_STORE_NONE)
+            {
+              g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                           "Multiple store strings");
+              return;
+            }
+
+          if (text_equals (text, text_len, "system"))
+            {
+              store = GF_CONFIG_STORE_SYSTEM;
+            }
+          else if (text_equals (text, text_len, "user"))
+            {
+              store = GF_CONFIG_STORE_USER;
+            }
+          else
+            {
+              g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                           "Invalid store %.*s", (int) text_len, text);
+              return;
+            }
+
+          parser->pending_store = store;
+          return;
+        }
+
+      case STATE_DBUS:
+        {
+          parser->enable_dbus_set = TRUE;
+          read_bool (text, text_len,
+                     &parser->enable_dbus,
+                     error);
+          return;
+        }
+
       default:
         break;
     }
@@ -1125,6 +1433,7 @@ static gboolean
 read_config_file (GfMonitorConfigStore  *config_store,
                   GFile                 *file,
                   GfMonitorsConfigFlag   extra_config_flags,
+                  GHashTable           **out_configs,
                   GError               **error)
 {
   gchar *buffer;
@@ -1140,8 +1449,15 @@ read_config_file (GfMonitorConfigStore  *config_store,
 
   parser = (ConfigParser) {
     .state = STATE_INITIAL,
+    .file = file,
     .config_store = config_store,
-    .extra_config_flags = extra_config_flags
+    .pending_configs = g_hash_table_new_full (gf_monitors_config_key_hash,
+                                              gf_monitors_config_key_equal,
+                                              NULL,
+                                              g_object_unref),
+    .extra_config_flags = extra_config_flags,
+    .unknown_state_root = -1,
+    .pending_store = GF_CONFIG_STORE_NONE
   };
 
   parse_context = g_markup_parse_context_new (&config_parser,
@@ -1161,12 +1477,18 @@ read_config_file (GfMonitorConfigStore  *config_store,
       g_clear_pointer (&parser.current_logical_monitor_config,
                        gf_logical_monitor_config_free);
 
+      g_list_free (parser.stores);
+      g_hash_table_unref (parser.pending_configs);
+
       g_markup_parse_context_free (parse_context);
       g_free (buffer);
 
       return FALSE;
     }
 
+  *out_configs = parser.pending_configs;
+  parser.pending_configs = NULL;
+
   g_markup_parse_context_free (parse_context);
   g_free (buffer);
 
@@ -1464,6 +1786,11 @@ gf_monitor_config_store_save (GfMonitorConfigStore *config_store)
       return;
     }
 
+  if (config_store->has_stores_policy &&
+      !g_list_find (config_store->stores_policy,
+                    GINT_TO_POINTER (GF_CONFIG_STORE_USER)))
+    return;
+
   config_store->save_cancellable = g_cancellable_new ();
 
   buffer = generate_config_xml (config_store);
@@ -1498,85 +1825,12 @@ static void
 gf_monitor_config_store_constructed (GObject *object)
 {
   GfMonitorConfigStore *config_store;
-  const char * const *system_dirs;
-  char *user_file_path;
-  GError *error;
 
   G_OBJECT_CLASS (gf_monitor_config_store_parent_class)->constructed (object);
 
   config_store = GF_MONITOR_CONFIG_STORE (object);
-  error = NULL;
 
-  for (system_dirs = g_get_system_config_dirs ();
-       system_dirs && *system_dirs;
-       system_dirs++)
-    {
-      char *system_file_path;
-
-      system_file_path = g_build_filename (*system_dirs, "monitors.xml", NULL);
-
-      if (g_file_test (system_file_path, G_FILE_TEST_EXISTS))
-        {
-          GFile *system_file;
-
-          system_file = g_file_new_for_path (system_file_path);
-
-          if (!read_config_file (config_store,
-                                 system_file,
-                                 GF_MONITORS_CONFIG_FLAG_SYSTEM_CONFIG,
-                                 &error))
-            {
-              if (g_error_matches (error,
-                                   GF_MONITOR_CONFIG_STORE_ERROR,
-                                   GF_MONITOR_CONFIG_STORE_ERROR_NEEDS_MIGRATION))
-                g_warning ("System monitor configuration file (%s) is "
-                           "incompatible; ask your administrator to migrate "
-                           "the system monitor configuration.",
-                           system_file_path);
-              else
-                g_warning ("Failed to read monitors config file '%s': %s",
-                           system_file_path, error->message);
-
-              g_clear_error (&error);
-            }
-
-          g_object_unref (system_file);
-        }
-
-      g_free (system_file_path);
-    }
-
-  user_file_path = g_build_filename (g_get_user_config_dir (), "monitors.xml", NULL);
-  config_store->user_file = g_file_new_for_path (user_file_path);
-
-  if (g_file_test (user_file_path, G_FILE_TEST_EXISTS))
-    {
-      if (!read_config_file (config_store,
-                             config_store->user_file,
-                             GF_MONITORS_CONFIG_FLAG_NONE,
-                             &error))
-        {
-          if (error->domain == GF_MONITOR_CONFIG_STORE_ERROR &&
-              error->code == GF_MONITOR_CONFIG_STORE_ERROR_NEEDS_MIGRATION)
-            {
-              g_clear_error (&error);
-              if (!gf_migrate_old_user_monitors_config (config_store, &error))
-                {
-                  g_warning ("Failed to migrate old monitors config file: %s",
-                             error->message);
-                  g_error_free (error);
-                }
-            }
-          else
-            {
-              g_warning ("Failed to read monitors config file '%s': %s",
-                         user_file_path, error->message);
-              g_error_free (error);
-            }
-        }
-    }
-
-  g_free (user_file_path);
+  gf_monitor_config_store_reset (config_store);
 }
 
 static void
@@ -1601,6 +1855,7 @@ gf_monitor_config_store_dispose (GObject *object)
   g_clear_object (&config_store->user_file);
   g_clear_object (&config_store->custom_read_file);
   g_clear_object (&config_store->custom_write_file);
+  g_clear_pointer (&config_store->stores_policy, g_list_free);
 
   G_OBJECT_CLASS (gf_monitor_config_store_parent_class)->dispose (object);
 }
@@ -1680,6 +1935,8 @@ gf_monitor_config_store_init (GfMonitorConfigStore *config_store)
   config_store->configs = g_hash_table_new_full (gf_monitors_config_key_hash,
                                                  gf_monitors_config_key_equal,
                                                  NULL, g_object_unref);
+
+  config_store->policy.enable_dbus = TRUE;
 }
 
 GfMonitorConfigStore *
@@ -1721,20 +1978,36 @@ gboolean
 gf_monitor_config_store_set_custom (GfMonitorConfigStore  *config_store,
                                     const gchar           *read_path,
                                     const gchar           *write_path,
+                                    GfMonitorsConfigFlag   config_flags,
                                     GError               **error)
 {
+  GHashTable *new_configs;
+
+  new_configs = NULL;
+
   g_clear_object (&config_store->custom_read_file);
   g_clear_object (&config_store->custom_write_file);
-  g_hash_table_remove_all (config_store->configs);
 
   config_store->custom_read_file = g_file_new_for_path (read_path);
   if (write_path)
     config_store->custom_write_file = g_file_new_for_path (write_path);
 
-  return read_config_file (config_store,
-                           config_store->custom_read_file,
-                           GF_MONITORS_CONFIG_FLAG_NONE,
-                           error);
+  g_clear_pointer (&config_store->stores_policy, g_list_free);
+  config_store->has_stores_policy = FALSE;
+  config_store->policy.enable_dbus = TRUE;
+  config_store->has_dbus_policy = FALSE;
+
+  if (!read_config_file (config_store,
+                         config_store->custom_read_file,
+                         config_flags,
+                         &new_configs,
+                         error))
+    return FALSE;
+
+  g_clear_pointer (&config_store->configs, g_hash_table_unref);
+  config_store->configs = new_configs;
+
+  return TRUE;
 }
 
 gint
@@ -1748,3 +2021,141 @@ gf_monitor_config_store_get_monitor_manager (GfMonitorConfigStore *config_store)
 {
   return config_store->monitor_manager;
 }
+
+void
+gf_monitor_config_store_reset (GfMonitorConfigStore  *config_store)
+{
+  GHashTable *system_configs;
+  GHashTable *user_configs;
+  const char * const *system_dirs;
+  char *user_file_path;
+  GError *error;
+
+  system_configs = NULL;
+  user_configs = NULL;
+  error = NULL;
+
+  g_clear_object (&config_store->user_file);
+  g_clear_object (&config_store->custom_read_file);
+  g_clear_object (&config_store->custom_write_file);
+  g_hash_table_remove_all (config_store->configs);
+
+  for (system_dirs = g_get_system_config_dirs ();
+       system_dirs && *system_dirs;
+       system_dirs++)
+    {
+      char *system_file_path;
+
+      system_file_path = g_build_filename (*system_dirs, "monitors.xml", NULL);
+
+      if (g_file_test (system_file_path, G_FILE_TEST_EXISTS))
+        {
+          GFile *system_file;
+
+          system_file = g_file_new_for_path (system_file_path);
+
+          if (!read_config_file (config_store,
+                                 system_file,
+                                 GF_MONITORS_CONFIG_FLAG_SYSTEM_CONFIG,
+                                 &system_configs,
+                                 &error))
+            {
+              if (g_error_matches (error,
+                                   GF_MONITOR_CONFIG_STORE_ERROR,
+                                   GF_MONITOR_CONFIG_STORE_ERROR_NEEDS_MIGRATION))
+                g_warning ("System monitor configuration file (%s) is "
+                           "incompatible; ask your administrator to migrate "
+                           "the system monitor configuration.",
+                           system_file_path);
+              else
+                g_warning ("Failed to read monitors config file '%s': %s",
+                           system_file_path, error->message);
+
+              g_clear_error (&error);
+            }
+
+          g_object_unref (system_file);
+        }
+
+      g_free (system_file_path);
+    }
+
+  user_file_path = g_build_filename (g_get_user_config_dir (), "monitors.xml", NULL);
+  config_store->user_file = g_file_new_for_path (user_file_path);
+
+  if (g_file_test (user_file_path, G_FILE_TEST_EXISTS))
+    {
+      if (!read_config_file (config_store,
+                             config_store->user_file,
+                             GF_MONITORS_CONFIG_FLAG_NONE,
+                             &user_configs,
+                             &error))
+        {
+          if (error->domain == GF_MONITOR_CONFIG_STORE_ERROR &&
+              error->code == GF_MONITOR_CONFIG_STORE_ERROR_NEEDS_MIGRATION)
+            {
+              g_clear_error (&error);
+              if (!gf_migrate_old_user_monitors_config (config_store, &error))
+                {
+                  g_warning ("Failed to migrate old monitors config file: %s",
+                             error->message);
+                  g_error_free (error);
+                }
+            }
+          else
+            {
+              g_warning ("Failed to read monitors config file '%s': %s",
+                         user_file_path, error->message);
+              g_error_free (error);
+            }
+        }
+    }
+
+  if (config_store->has_stores_policy)
+    {
+      GList *l;
+
+      for (l = g_list_last (config_store->stores_policy); l; l = l->prev)
+        {
+          GfConfigStore store;
+
+          store = GPOINTER_TO_INT (l->data);
+
+          switch (store)
+            {
+              case GF_CONFIG_STORE_SYSTEM:
+                if (system_configs)
+                  replace_configs (config_store, system_configs);
+                break;
+
+              case GF_CONFIG_STORE_USER:
+                if (user_configs)
+                  replace_configs (config_store, user_configs);
+                break;
+
+              case GF_CONFIG_STORE_NONE:
+              default:
+                g_assert_not_reached ();
+                break;
+            }
+        }
+    }
+  else
+    {
+      if (system_configs)
+        replace_configs (config_store, system_configs);
+
+      if (user_configs)
+        replace_configs (config_store, user_configs);
+    }
+
+  g_clear_pointer (&system_configs, g_hash_table_unref);
+  g_clear_pointer (&user_configs, g_hash_table_unref);
+  g_free (user_file_path);
+}
+
+const GfMonitorConfigPolicy *
+gf_monitor_config_store_get_policy (GfMonitorConfigStore *config_store)
+{
+  return &config_store->policy;
+}
