@@ -32,9 +32,8 @@ G_BEGIN_DECLS
 typedef enum
 {
   GF_MONITOR_MANAGER_CAPABILITY_NONE = 0,
-  GF_MONITOR_MANAGER_CAPABILITY_MIRRORING = (1 << 0),
-  GF_MONITOR_MANAGER_CAPABILITY_LAYOUT_MODE = (1 << 1),
-  GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED = (1 << 2)
+  GF_MONITOR_MANAGER_CAPABILITY_LAYOUT_MODE = (1 << 0),
+  GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED = (1 << 1)
 } GfMonitorManagerCapability;
 
 /* Equivalent to the 'method' enum in org.gnome.Mutter.DisplayConfig */
@@ -74,29 +73,6 @@ typedef enum
   GF_CONNECTOR_TYPE_DSI = 16
 } GfConnectorType;
 
-/* Same as KMS mode flags and X11 randr flags */
-typedef enum
-{
-  GF_CRTC_MODE_FLAG_NONE = 0,
-
-  GF_CRTC_MODE_FLAG_PHSYNC = (1 << 0),
-  GF_CRTC_MODE_FLAG_NHSYNC = (1 << 1),
-  GF_CRTC_MODE_FLAG_PVSYNC = (1 << 2),
-  GF_CRTC_MODE_FLAG_NVSYNC = (1 << 3),
-  GF_CRTC_MODE_FLAG_INTERLACE = (1 << 4),
-  GF_CRTC_MODE_FLAG_DBLSCAN = (1 << 5),
-  GF_CRTC_MODE_FLAG_CSYNC = (1 << 6),
-  GF_CRTC_MODE_FLAG_PCSYNC = (1 << 7),
-  GF_CRTC_MODE_FLAG_NCSYNC = (1 << 8),
-  GF_CRTC_MODE_FLAG_HSKEW = (1 << 9),
-  GF_CRTC_MODE_FLAG_BCAST = (1 << 10),
-  GF_CRTC_MODE_FLAG_PIXMUX = (1 << 11),
-  GF_CRTC_MODE_FLAG_DBLCLK = (1 << 12),
-  GF_CRTC_MODE_FLAG_CLKDIV2 = (1 << 13),
-
-  GF_CRTC_MODE_FLAG_MASK = 0x3fff
-} GfCrtcModeFlag;
-
 G_END_DECLS
 
 #endif
