@@ -26,7 +26,6 @@
 #define GF_MONITOR_MANAGER_PRIVATE_H
 
 #include <libgnome-desktop/gnome-pnp-ids.h>
-#include <libupower-glib/upower.h>
 
 #include "gf-backend-private.h"
 #include "gf-dbus-display-config.h"
@@ -70,8 +69,6 @@ struct _GfMonitorManager
   GfMonitorConfigManager      *config_manager;
 
   GnomePnpIds                 *pnp_ids;
-  UpClient                    *up_client;
-  gboolean                     lid_is_closed;
 
   GfMonitorSwitchConfigType    current_switch_config;
 };
@@ -83,8 +80,6 @@ typedef struct
   GBytes                     * (* read_edid)                    (GfMonitorManager            *manager,
                                                                  GfOutput                    *output);
 
-  gboolean                     (* is_lid_closed)                (GfMonitorManager            *manager);
-
   void                         (* read_current_state)           (GfMonitorManager            *manager);
 
   void                         (* ensure_initial_config)        (GfMonitorManager            *manager);
@@ -126,6 +121,7 @@ typedef struct
                                                                  GfMonitorTransform           transform);
 
   gfloat                       (* calculate_monitor_mode_scale) (GfMonitorManager            *manager,
+                                                                 GfLogicalMonitorLayoutMode   layout_mode,
                                                                  GfMonitor                   *monitor,
                                                                  GfMonitorMode               *monitor_mode);
 
@@ -191,9 +187,8 @@ GfMonitorsConfig           *gf_monitor_manager_ensure_configured            (GfM
 void                        gf_monitor_manager_update_logical_state_derived (GfMonitorManager            *manager,
                                                                              GfMonitorsConfig            *config);
 
-gboolean                    gf_monitor_manager_is_lid_closed                (GfMonitorManager            *manager);
-
 gfloat                      gf_monitor_manager_calculate_monitor_mode_scale (GfMonitorManager            *manager,
+                                                                             GfLogicalMonitorLayoutMode   layout_mode,
                                                                              GfMonitor                   *monitor,
                                                                              GfMonitorMode               *monitor_mode);
 
@@ -227,6 +222,9 @@ GfPowerSave                 gf_monitor_manager_get_power_save_mode          (GfM
 void                        gf_monitor_manager_power_save_mode_changed      (GfMonitorManager            *manager,
                                                                              GfPowerSave                  mode);
 
+gboolean                    gf_monitor_manager_is_monitor_visible           (GfMonitorManager            *manager,
+                                                                             GfMonitor                   *monitor);
+
 G_END_DECLS
 
 #endif
