@@ -34,6 +34,7 @@
 #include "gf-settings-private.h"
 
 #define SCALE_FACTORS_PER_INTEGER 4
+#define SCALE_FACTORS_STEPS (1.0f / (float) SCALE_FACTORS_PER_INTEGER)
 #define MINIMUM_SCALE_FACTOR 1.0f
 #define MAXIMUM_SCALE_FACTOR 4.0f
 #define MINIMUM_LOGICAL_AREA (800 * 480)
@@ -226,8 +227,9 @@ gf_monitor_mode_spec_equals (GfMonitorModeSpec *spec,
 }
 
 static float
-calculate_scale (GfMonitor     *monitor,
-                 GfMonitorMode *monitor_mode)
+calculate_scale (GfMonitor                 *monitor,
+                 GfMonitorMode             *monitor_mode,
+                 GfMonitorScalesConstraint  constraints)
 {
   gint resolution_width, resolution_height;
   gint width_mm, height_mm;
@@ -240,7 +242,7 @@ calculate_scale (GfMonitor     *monitor,
                                   &resolution_height);
 
   if (resolution_height < HIDPI_MIN_HEIGHT)
-    goto out;
+    return scale;
 
   /* 4K TV */
   switch (gf_monitor_get_connector_type (monitor))
@@ -248,7 +250,7 @@ calculate_scale (GfMonitor     *monitor,
       case GF_CONNECTOR_TYPE_HDMIA:
       case GF_CONNECTOR_TYPE_HDMIB:
         if (resolution_width < SMALLEST_4K_WIDTH)
-          goto out;
+          return scale;
         break;
 
       case GF_CONNECTOR_TYPE_Unknown:
@@ -276,7 +278,7 @@ calculate_scale (GfMonitor     *monitor,
    * physical size.
    */
   if (gf_monitor_has_aspect_as_size (monitor))
-    goto out;
+    return scale;
 
   if (width_mm > 0 && height_mm > 0)
     {
@@ -293,7 +295,6 @@ calculate_scale (GfMonitor     *monitor,
         scale = 2.0;
     }
 
-out:
   return scale;
 }
 
@@ -304,40 +305,45 @@ is_logical_size_large_enough (gint width,
   return width * height >= MINIMUM_LOGICAL_AREA;
 }
 
+static gboolean
+is_scale_valid_for_size (float width,
+                         float height,
+                         float scale)
+{
+  if (scale < MINIMUM_SCALE_FACTOR || scale > MAXIMUM_SCALE_FACTOR)
+    return FALSE;
+
+  return is_logical_size_large_enough (floorf (width / scale),
+                                       floorf (height / scale));
+}
+
 static gfloat
-get_closest_scale_factor_for_resolution (gfloat width,
-                                         gfloat height,
-                                         gfloat scale,
-                                         gfloat scale_step)
+get_closest_scale_factor_for_resolution (float width,
+                                         float height,
+                                         float scale,
+                                         float threshold)
 {
   guint i, j;
   gfloat scaled_h;
   gfloat scaled_w;
   gfloat best_scale;
   gint base_scaled_w;
-  gboolean limit_exceeded;
   gboolean found_one;
 
   best_scale = 0;
-  scaled_w = width / scale;
-  scaled_h = height / scale;
 
-  if (scale < MINIMUM_SCALE_FACTOR ||
-      scale > MAXIMUM_SCALE_FACTOR ||
-      !is_logical_size_large_enough (floorf (scaled_w), floorf (scaled_h)))
-    goto out;
+  if (!is_scale_valid_for_size (width, height, scale))
+    return best_scale;
 
-  if (floorf (scaled_w) == scaled_w && floorf (scaled_h) == scaled_h)
+  if (fmodf (width, scale) == 0.0f && fmodf (height, scale) == 0.0f)
     return scale;
 
   i = 0;
   found_one = FALSE;
-  limit_exceeded = FALSE;
-  base_scaled_w = floorf (scaled_w);
+  base_scaled_w = floorf (width / scale);
 
   do
     {
-
       for (j = 0; j < 2; j++)
         {
           gfloat current_scale;
@@ -347,13 +353,12 @@ get_closest_scale_factor_for_resolution (gfloat width,
           current_scale = width / scaled_w;
           scaled_h = height / current_scale;
 
-          if (current_scale >= scale + scale_step ||
-              current_scale <= scale - scale_step ||
+          if (current_scale >= scale + threshold ||
+              current_scale <= scale - threshold ||
               current_scale < MINIMUM_SCALE_FACTOR ||
               current_scale > MAXIMUM_SCALE_FACTOR)
             {
-              limit_exceeded = TRUE;
-              continue;
+              return best_scale;
             }
 
           if (floorf (scaled_h) == scaled_h)
@@ -367,9 +372,8 @@ get_closest_scale_factor_for_resolution (gfloat width,
 
       i++;
     }
-  while (!found_one && !limit_exceeded);
+  while (!found_one);
 
-out:
   return best_scale;
 }
 
@@ -1052,8 +1056,9 @@ gf_monitor_calculate_crtc_pos (GfMonitor          *monitor,
 }
 
 gfloat
-gf_monitor_calculate_mode_scale (GfMonitor     *monitor,
-                                 GfMonitorMode *monitor_mode)
+gf_monitor_calculate_mode_scale (GfMonitor                 *monitor,
+                                 GfMonitorMode             *monitor_mode,
+                                 GfMonitorScalesConstraint  constraints)
 {
   GfMonitorPrivate *priv;
   GfMonitorManager *monitor_manager;
@@ -1070,7 +1075,7 @@ gf_monitor_calculate_mode_scale (GfMonitor     *monitor,
   if (gf_settings_get_global_scaling_factor (settings, &global_scaling_factor))
     return global_scaling_factor;
 
-  return calculate_scale (monitor, monitor_mode);
+  return calculate_scale (monitor, monitor_mode, constraints);
 }
 
 gfloat *
@@ -1081,10 +1086,8 @@ gf_monitor_calculate_supported_scales (GfMonitor                 *monitor,
 {
   guint i, j;
   gint width, height;
-  gfloat scale_steps;
   GArray *supported_scales;
 
-  scale_steps = 1.0f / SCALE_FACTORS_PER_INTEGER;
   supported_scales = g_array_new (FALSE, FALSE, sizeof (gfloat));
 
   gf_monitor_mode_get_resolution (monitor_mode, &width, &height);
@@ -1093,24 +1096,37 @@ gf_monitor_calculate_supported_scales (GfMonitor                 *monitor,
        i <= ceilf (MAXIMUM_SCALE_FACTOR);
        i++)
     {
-      for (j = 0; j < SCALE_FACTORS_PER_INTEGER; j++)
+      if (constraints & GF_MONITOR_SCALES_CONSTRAINT_NO_FRAC)
         {
-          gfloat scale;
-          gfloat scale_value = i + j * scale_steps;
-
-          if ((constraints & GF_MONITOR_SCALES_CONSTRAINT_NO_FRAC) &&
-              fmodf (scale_value, 1.0) != 0.0f)
+          if (is_scale_valid_for_size (width, height, i))
             {
-              continue;
+              float scale = i;
+              g_array_append_val (supported_scales, scale);
             }
+        }
+      else
+        {
+          float max_bound;
 
-          scale = get_closest_scale_factor_for_resolution (width,
-                                                           height,
-                                                           scale_value,
-                                                           scale_steps);
+          if (i == floorf (MINIMUM_SCALE_FACTOR) ||
+              i == ceilf (MAXIMUM_SCALE_FACTOR))
+            max_bound = SCALE_FACTORS_STEPS;
+          else
+            max_bound = SCALE_FACTORS_STEPS / 2.0f;
 
-          if (scale > 0.0f)
-            g_array_append_val (supported_scales, scale);
+          for (j = 0; j < SCALE_FACTORS_PER_INTEGER; j++)
+            {
+              gfloat scale;
+              gfloat scale_value = i + j * SCALE_FACTORS_STEPS;
+
+              scale = get_closest_scale_factor_for_resolution (width,
+                                                               height,
+                                                               scale_value,
+                                                               max_bound);
+
+              if (scale > 0.0f)
+                g_array_append_val (supported_scales, scale);
+            }
         }
     }
 
