@@ -155,7 +155,7 @@ get_edid_property (Display  *xdisplay,
 
   if (actual_type == XA_INTEGER && actual_format == 8)
     {
-      result = g_memdup (prop, nitems);
+      result = g_memdup2 (prop, nitems);
       if (len)
         *len = nitems;
     }
@@ -852,7 +852,7 @@ gf_output_xrandr_new (GfGpuXrandr   *gpu_xrandr,
       output_info->height_mm = xrandr_output->mm_height;
     }
 
-  if (!gf_monitor_manager_xrandr_has_randr15 (monitor_manager_xrandr))
+  if (gf_monitor_manager_xrandr_has_randr15 (monitor_manager_xrandr))
     output_info_init_tile_info (output_info, xdisplay, output_id);
 
   output_info_init_modes (output_info, gpu, xrandr_output);
@@ -875,7 +875,7 @@ gf_output_xrandr_new (GfGpuXrandr   *gpu_xrandr,
   output_info_init_backlight_limits_xrandr (output_info, xdisplay, output_id);
 
   output = g_object_new (GF_TYPE_OUTPUT_XRANDR,
-                         "id", output_id,
+                         "id", (uint64_t) output_id,
                          "gpu", gpu_xrandr,
                          "info", output_info,
                          NULL);
