@@ -29,6 +29,8 @@
 #include "config.h"
 #include "gf-output-private.h"
 
+#include "gf-crtc-private.h"
+
 typedef struct
 {
   uint64_t      id;
@@ -37,6 +39,8 @@ typedef struct
 
   GfOutputInfo *info;
 
+  GfMonitor    *monitor;
+
   /* The CRTC driving this output, NULL if the output is not enabled */
   GfCrtc       *crtc;
 
@@ -240,6 +244,31 @@ gf_output_get_info (GfOutput *self)
   return priv->info;
 }
 
+GfMonitor *
+gf_output_get_monitor (GfOutput *self)
+{
+  GfOutputPrivate *priv;
+
+  priv = gf_output_get_instance_private (self);
+
+  g_warn_if_fail (priv->monitor);
+
+  return priv->monitor;
+}
+
+void
+gf_output_set_monitor (GfOutput  *self,
+                       GfMonitor *monitor)
+{
+  GfOutputPrivate *priv;
+
+  priv = gf_output_get_instance_private (self);
+
+  g_warn_if_fail (!priv->monitor);
+
+  priv->monitor = monitor;
+}
+
 const char *
 gf_output_get_name (GfOutput *self)
 {
@@ -251,7 +280,7 @@ gf_output_get_name (GfOutput *self)
 }
 
 void
-gf_output_assign_crtc (GfOutput                *self,
+gf_output_assign_crtc (GfOutput                 *self,
                        GfCrtc                   *crtc,
                        const GfOutputAssignment *output_assignment)
 {
@@ -261,8 +290,12 @@ gf_output_assign_crtc (GfOutput                *self,
 
   g_assert (crtc);
 
+  gf_output_unassign_crtc (self);
+
   g_set_object (&priv->crtc, crtc);
 
+  gf_crtc_assign_output (crtc, self);
+
   priv->is_primary = output_assignment->is_primary;
   priv->is_presentation = output_assignment->is_presentation;
   priv->is_underscanning = output_assignment->is_underscanning;
@@ -275,7 +308,11 @@ gf_output_unassign_crtc (GfOutput *output)
 
   priv = gf_output_get_instance_private (output);
 
-  g_clear_object (&priv->crtc);
+  if (priv->crtc != NULL)
+    {
+      gf_crtc_unassign_output (priv->crtc, output);
+      g_clear_object (&priv->crtc);
+    }
 
   priv->is_primary = FALSE;
   priv->is_presentation = FALSE;
