@@ -291,8 +291,10 @@ cleanup_cache_for_monitor (gchar *cache_dir,
 		path = g_build_filename (cache_dir, file, NULL);
 		/* purge files with same monitor id */
 		if (g_str_has_prefix (file, monitor_prefix) &&
-		    g_file_test (path, G_FILE_TEST_IS_REGULAR))
-			g_unlink (path);
+		    g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
+			if (g_unlink (path) != 0)
+				g_warning ("Failed to delete %s", path);
+		}
 
 		g_free (path);
 
@@ -343,7 +345,8 @@ refresh_cache_file (GfBG      *bg,
 
 		if (format != NULL) {
 			if (!g_file_test (cache_dir, G_FILE_TEST_IS_DIR)) {
-				g_mkdir_with_parents (cache_dir, 0700);
+				if (g_mkdir_with_parents (cache_dir, 0700) != 0)
+					g_warning ("Failed to mkdir %s", cache_dir);
 			} else {
 				cleanup_cache_for_monitor (cache_dir, num_monitor);
 			}
@@ -389,7 +392,8 @@ draw_color_area (GfBG         *bg,
         extent.width = gdk_pixbuf_get_width (dest);
         extent.height = gdk_pixbuf_get_height (dest);
 
-        gdk_rectangle_intersect (rect, &extent, rect);
+        if (!gdk_rectangle_intersect (rect, &extent, rect))
+                return;
 	
 	switch (bg->color_type) {
 	case G_DESKTOP_BACKGROUND_SHADING_SOLID:
@@ -1241,12 +1245,12 @@ pixbuf_average_value (GdkPixbuf *pixbuf,
 				a = *p++;
 				
 				a_total += a;
-				r_total += r * a;
-				g_total += g * a;
-				b_total += b * a;
+				r_total += (guint64) r * a;
+				g_total += (guint64) g * a;
+				b_total += (guint64) b * a;
 			}
 		}
-		dividend = height * width * 0xFF;
+		dividend = (guint64) height * width * 0xFF;
 		a_total *= 0xFF;
 	} else {
 		for (row = 0; row < height; row++) {
@@ -1261,7 +1265,7 @@ pixbuf_average_value (GdkPixbuf *pixbuf,
 				b_total += b;
 			}
 		}
-		dividend = height * width;
+		dividend = (guint64) height * width;
 		a_total = dividend * 0xFF;
 	}
 
