@@ -76,7 +76,7 @@ destroy_dbus_watch (gpointer data)
   g_free (watch->dbus_name);
   g_bus_unwatch_name (watch->name_watcher_id);
 
-  g_slice_free (DBusWatch, watch);
+  g_free (watch);
 }
 
 static void
@@ -125,7 +125,7 @@ make_dbus_watch (MetaDBusIdleMonitor   *skeleton,
   connection = g_dbus_method_invocation_get_connection (invocation);
   sender = g_dbus_method_invocation_get_sender (invocation);
 
-  watch = g_slice_new (DBusWatch);
+  watch = g_new0 (DBusWatch, 1);
   watch->dbus_monitor = g_object_ref (skeleton);
   watch->monitor = g_object_ref (monitor);
   watch->dbus_name = g_strdup (sender);
@@ -196,7 +196,7 @@ create_monitor_skeleton (GDBusObjectManagerServer *server,
                          const gchar              *path)
 {
   MetaDBusIdleMonitor *skeleton;
-  MetaDBusObjectSkeleton *object;
+  GDBusObjectSkeleton *object;
 
   skeleton = meta_dbus_idle_monitor_skeleton_new ();
   g_signal_connect_object (skeleton, "handle-add-idle-watch",
@@ -208,10 +208,11 @@ create_monitor_skeleton (GDBusObjectManagerServer *server,
   g_signal_connect_object (skeleton, "handle-get-idletime",
                            G_CALLBACK (handle_get_idletime), monitor, 0);
 
-  object = meta_dbus_object_skeleton_new (path);
-  meta_dbus_object_skeleton_set_idle_monitor (object, skeleton);
+  object = g_dbus_object_skeleton_new (path);
+  g_dbus_object_skeleton_add_interface (object,
+                                        G_DBUS_INTERFACE_SKELETON (skeleton));
 
-  g_dbus_object_manager_server_export (server, G_DBUS_OBJECT_SKELETON (object));
+  g_dbus_object_manager_server_export (server, object);
 
   g_object_unref (skeleton);
   g_object_unref (object);
