@@ -350,7 +350,10 @@ get_user_icon (char *username)
       pixbuf = gdk_pixbuf_new_from_file_at_size (icon_filename, 16, 16, &error);
       if (pixbuf == NULL)
         {
-          g_warning ("Couldn't open user icon: %s", error->message);
+          if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+            g_warning ("Couldn't open user icon: %s", error->message);
+          else
+            g_debug ("Couldn't open user icon: %s", error->message);
           g_error_free (error);
         }
     }
@@ -494,11 +497,15 @@ setup_users_combobox (FlashbackPolkitDialog *dialog)
 
   if (g_strv_length (dialog->users) > 1)
     {
+      gboolean sensitive;
+
       setup_users_store (dialog);
 
-      gtk_widget_set_sensitive (dialog->prompt_label, FALSE);
-      gtk_widget_set_sensitive (dialog->password_entry, FALSE);
-      gtk_widget_set_sensitive (dialog->auth_button, FALSE);
+      sensitive = dialog->selected_user != NULL ? TRUE : FALSE;
+
+      gtk_widget_set_sensitive (dialog->prompt_label, sensitive);
+      gtk_widget_set_sensitive (dialog->password_entry, sensitive);
+      gtk_widget_set_sensitive (dialog->auth_button, sensitive);
     }
   else
     {
@@ -898,6 +905,20 @@ flashback_polkit_dialog_close (FlashbackPolkitDialog *dialog,
   gtk_window_close (GTK_WINDOW (dialog));
 }
 
+static gboolean
+delete_event_cb (FlashbackPolkitDialog *dialog,
+                 GdkEvent              *event,
+                 gpointer               user_data)
+{
+  if (!dialog->is_running)
+    return FALSE;
+
+  dialog->response = GTK_RESPONSE_DELETE_EVENT;
+  gtk_main_quit ();
+
+  return TRUE;
+}
+
 static void
 flashback_polkit_dialog_init (FlashbackPolkitDialog *dialog)
 {
@@ -914,6 +935,10 @@ flashback_polkit_dialog_init (FlashbackPolkitDialog *dialog)
   g_signal_connect (dialog, "close",
                     G_CALLBACK (flashback_polkit_dialog_close),
                     NULL);
+
+  g_signal_connect (dialog, "delete-event",
+                    G_CALLBACK (delete_event_cb),
+                    NULL);
 }
 
 /**
