@@ -23,43 +23,44 @@
 #include "config.h"
 #include "gf-manager.h"
 
-#include <libgnome-desktop/gnome-bg.h>
-
 #include "gf-window.h"
+#include "libcommon/gf-bg.h"
 
 struct _GfManager
 {
-  GObject         parent;
+  GObject           parent;
 
-  GSettings      *settings;
-  GnomeBG        *bg;
+  GSettings        *settings;
+  GfBG             *bg;
 
-  GfGrab         *grab;
-  GfFade         *fade;
+  GfGrab           *grab;
+  GfFade           *fade;
 
-  GfInputSources *input_sources;
+  GfMonitorManager *monitor_manager;
 
-  GSList         *windows;
+  GfInputSources   *input_sources;
 
-  gboolean        active;
+  GSList           *windows;
 
-  gboolean        lock_active;
+  gboolean          active;
 
-  gboolean        lock_enabled;
+  gboolean          lock_active;
 
-  glong           lock_timeout;
+  gboolean          lock_enabled;
 
-  gboolean        user_switch_enabled;
+  glong             lock_timeout;
 
-  time_t          activate_time;
+  gboolean          user_switch_enabled;
 
-  guint           lock_timeout_id;
-  guint           unfade_timeout_id;
+  time_t            activate_time;
 
-  gboolean        dialog_up;
+  guint             lock_timeout_id;
+  guint             unfade_timeout_id;
 
-  gulong          monitor_added_id;
-  gulong          monitor_removed_id;
+  gboolean          dialog_up;
+
+  gulong            monitor_added_id;
+  gulong            monitor_removed_id;
 };
 
 enum
@@ -221,25 +222,18 @@ apply_background (GfManager *self,
 
   g_debug ("Creating background: size %dx%d", rect.width, rect.height);
 
-#ifdef HAVE_GNOME_DESKTOP_3_35_4
-  surface = gnome_bg_create_surface (self->bg,
-                                     gtk_widget_get_window (GTK_WIDGET (window)),
-                                     rect.width,
-                                     rect.height);
-#else
-  surface = gnome_bg_create_surface (self->bg,
-                                     gtk_widget_get_window (GTK_WIDGET (window)),
-                                     rect.width,
-                                     rect.height,
-                                     FALSE);
-#endif
+  surface = gf_bg_create_surface (self->bg,
+                                  gtk_widget_get_window (GTK_WIDGET (window)),
+                                  rect.width,
+                                  rect.height,
+                                  FALSE);
 
   gf_window_set_background (window, surface);
   cairo_surface_destroy (surface);
 }
 
 static void
-bg_changed_cb (GnomeBG   *bg,
+bg_changed_cb (GfBG      *bg,
                GfManager *self)
 {
   GSList *l;
@@ -256,7 +250,7 @@ settings_change_event_cb (GSettings *settings,
                           gint       n_keys,
                           GfManager *self)
 {
-  gnome_bg_load_from_preferences (self->bg, self->settings);
+  gf_bg_load_from_preferences (self->bg, self->settings);
 
   return FALSE;
 }
@@ -686,7 +680,7 @@ static void
 gf_manager_init (GfManager *self)
 {
   self->settings = g_settings_new ("org.gnome.desktop.screensaver");
-  self->bg = gnome_bg_new ();
+  self->bg = gf_bg_new ();
 
   g_signal_connect (self->bg, "changed",
                     G_CALLBACK (bg_changed_cb),
@@ -696,7 +690,7 @@ gf_manager_init (GfManager *self)
                     G_CALLBACK (settings_change_event_cb),
                     self);
 
-  gnome_bg_load_from_preferences (self->bg, self->settings);
+  gf_bg_load_from_preferences (self->bg, self->settings);
 }
 
 GfManager *
@@ -709,6 +703,13 @@ gf_manager_new (GfGrab *grab,
                        NULL);
 }
 
+void
+gf_manager_set_monitor_manager (GfManager        *self,
+                                GfMonitorManager *monitor_manager)
+{
+  self->monitor_manager = monitor_manager;
+}
+
 void
 gf_manager_set_input_sources (GfManager      *self,
                               GfInputSources *input_sources)
