@@ -19,18 +19,17 @@
 #define FLASHBACK_SHELL_H
 
 #include <glib-object.h>
+#include "backends/gf-monitor-manager.h"
 
 G_BEGIN_DECLS
 
-typedef struct _FlashbackDisplayConfig FlashbackDisplayConfig;
-
 #define FLASHBACK_TYPE_SHELL flashback_shell_get_type ()
 G_DECLARE_FINAL_TYPE (FlashbackShell, flashback_shell, FLASHBACK, SHELL, GObject)
 
-FlashbackShell *flashback_shell_new                (void);
+FlashbackShell *flashback_shell_new                 (void);
 
-void            flashback_shell_set_display_config (FlashbackShell         *shell,
-                                                    FlashbackDisplayConfig *config);
+void            flashback_shell_set_monitor_manager (FlashbackShell   *shell,
+                                                     GfMonitorManager *monitor_manager);
 
 G_END_DECLS
 
